#--------------------------------------------------------------------------------------
# Function:    Test_forced_oscillation_2A4M.1.py
# Usage:       Compare forced oscillation magnitude with and without battery (75MW)
# Author:      Lin Zhu
# Requirement: Python 2.7, PSS/E33 and python modules for psse
# Version:     v1.0 2019-10-18
# Description:
#   Mitigation of forced oscillation using utility-scale battery energy storage system
#   Demo on Two-area four-machine system  
#   Reference: Benchmark Systems for Small-Signal Stability Analysis and Control
#              http://www.sel.eesc.usp.br/ieee/index.htm
#--------------------------------------------------------------------------------------

import csv, math
import os
import sys

# Setting the system environment of psse, based on PSSE installed directory
PSSE_PATH = 'C:\\Program Files (x86)\\PTI\\PSSE33\\PSSBIN'

sys.path.append(PSSE_PATH)
os.environ['PATH'] +=';' +PSSE_PATH

# Define the filename of channel file, excel file and text file
# Don't lost the last '\\'. Use the '\\' instead of '/', otherwise the excel file cannnot be saved in your the defined directory. 
#caseDir = 'C:\\Alan works\\04 HVDC overlay study\\C DynamicStudy\\4G2A_50Hz\\DC400MW\\'
caseDir = 'C:\\Users\\lzhu12\\Desktop\\Two_Areas_PSS_E\\Demo_Forced_Oscillation\\'

# Initialize the PSSE setting
import psspy
ierr = psspy.psseinit(150000)
psspy.throwPsseExcption = True
import redirect
redirect.psse2py()
if ierr==0:
	print "PSSE is intialized successfully!"
else:
	exit()


# my_freq = 1  means 0.10Hz disturbance frequency
# my_freq = 2  means 0.15Hz disturbance frequency
# ... ...
# my_freq = 12 means 0.65Hz disturbance frequency
# ... ...
# my_freq = 39 means 2.00Hz disturbance frequency
my_freq = [7,12]  

dTime = 0.1  # sine wave injection cycle and control cycle (second)

# add sine wave to generator at bus 1, no. 1
dist_Gen_bus = [1] 
dist_Gen_ID = [1]
Mbase = [0.9]
dis_mag = 0.4        # Disturbance Magnitude

BESS_Bus = 2         # Battery location
gain = [0,-2000]     # control gain: 0: no control, negative gain to mitigate forced oscillation
Feedback = 7         # Feedback signal: Bus frequency   
ChnNO = 1            # added feedback signal to channel NO #1

_i = psspy.getdefaultint()
_f = psspy.getdefaultreal()

for kk in range(len(gain)):
	out_name = r"""Mag""" + str(dis_mag) + r"""_Gain""" + str(0-gain[kk]) + r"""_BESS_""" + str(BESS_Bus) +r"""_FB_""" + str(Feedback)

	for Machine_index in range(len(dist_Gen_bus)): 
		for Freq_index in range(len(my_freq)):
			Sine_file_name = str(my_freq[Freq_index])+'.txt'
			
			psspy.read(0,r"""Benchmark_4ger_33_2015.raw""")
			psspy.dyre_new([1,1,1,1],r"""Benchmark_4ger_ESST1A_noTGR_PSSori_2015.dyr""","","","")	
			
			# add governor model to disturbance generator
			psspy.add_plant_model(dist_Gen_bus[Machine_index],r"""1""",7,r"""TGOV1""",0,"",0,[],[],
			7,[20.0,0.1,99.0,-99.0,0.1,0.1,0.0])
			
			# add BESS to BESS bus and as no 2
			psspy.machine_data_2(BESS_Bus,str(2), [_i,_i,_i,_i,_i,1], [0,0,25,-25,50,-50,75,_f, 99999.0,_f,_f,_f,_f,_f,_f,_f,_f])
			
			if BESS_Bus == 1:
				psspy.dyre_add([204,75,10,17],r"""myBESS_1.dyr""","","")
			if BESS_Bus == 2:
				psspy.dyre_add([204,75,10,17],r"""myBESS_2.dyr""","","")
			if BESS_Bus == 3:
				psspy.dyre_add([204,75,10,17],r"""myBESS_3.dyr""","","")
			if BESS_Bus == 4:
				psspy.dyre_add([204,75,10,17],r"""myBESS_4.dyr""","","")

			psspy.fnsl([0,0,0,1,1,0,99,0])

			psspy.cong(0)
			psspy.conl(0,1,1,[0,0],[0.0, 100.0, 100.0, 1.0])
			psspy.conl(0,1,2,[0,0],[0.0, 100.0, 100.0, 1.0])
			psspy.conl(0,1,3,[0,0],[0.0, 100.0, 100.0, 1.0])
			
			# add channels
			psspy.delete_all_plot_channels()	
			psspy.bus_frequency_channel([-1,Feedback],r"""Bus_""" + str(Feedback) +r""""_Freq_""")  # feedback signal			
			psspy.chsb(0,1,[-1,-1,-1,1,12,0])   # add bus frequency 
			psspy.chsb(0,1,[-1,-1,-1,1,13,0])   # add bus voltage 
			psspy.chsb(0,1,[-1,-1,-1,1,15,0])   # add line active power  
			
			# disturbance genertor Pmech and Pelec
			psspy.machine_array_channel([-1,6,dist_Gen_bus[Machine_index]],str(dist_Gen_ID[Machine_index]),"")
			psspy.machine_array_channel([-1,2,dist_Gen_bus[Machine_index]],str(dist_Gen_ID[Machine_index]),"")
			
			# Battery Ip command, Auxiliary signal, and P output
			ierr, L = psspy.windmind(BESS_Bus,'2', 'WELEC','VAR')
			psspy.var_channel([-1,L+2],'BESS Ip Command')
			psspy.var_channel([-1,L+5],'BESS Auxiliary signal')
			psspy.machine_array_channel([-1,2,BESS_Bus],r"""2""","")  # BESS output
			
			psspy.lines_per_page_one_device(1,60)
			psspy.progress_output(2,str(dist_Gen_bus[Machine_index])+r"""_"""+str((my_freq[Freq_index]-1)*0.05+0.1) + r"""Hz_""" + out_name + r'.txt',[0,0])
			
			# read sine wave file
			print(' Read SineWave txt file ')
			with open(Sine_file_name,'r') as file_to_read:
				lines = file_to_read.readlines(-1)

			lines = [x.strip() for x in lines]
			lines = map(float, lines)
			print(len(lines))
			print(lines[5])

			psspy.strt_2([0,0], str(dist_Gen_bus[Machine_index])+r"""_""" +	str((my_freq[Freq_index]-1)*0.05+0.1) + r"""Hz_""" + out_name + r'.out')
			
			psspy.run(0,2.00,100,20,20)
			for i in range(len(lines)):
				
				if i == 0:
					# add disturbance
					psspy.increment_gref(dist_Gen_bus[Machine_index],str(dist_Gen_ID[Machine_index]),dis_mag*lines[i]/Mbase[Machine_index])	
					
					# Add auxiliary signal to Battery
					ierr, rval = psspy.chnval(ChnNO)
					ierr, L = psspy.windmind(BESS_Bus,'2', 'WELEC','VAR')
					ierr = psspy.change_var(L+5, rval*gain[kk])
					
					psspy.run(0, 2.00+dTime*(i+1),100,20,20)
				else:
					# add disturbance
					psspy.increment_gref(dist_Gen_bus[Machine_index],str(dist_Gen_ID[Machine_index]),-dis_mag*lines[i-1]/Mbase[Machine_index])
					
					psspy.increment_gref(dist_Gen_bus[Machine_index],str(dist_Gen_ID[Machine_index]),dis_mag*lines[i]/Mbase[Machine_index])	
					
					# Add auxiliary signal to Battery
					ierr, rval = psspy.chnval(ChnNO)
					ierr, L = psspy.windmind(BESS_Bus,'2', 'WELEC','VAR')
					ierr = psspy.change_var(L+5, rval*gain[kk])
	
					psspy.run(0, 2.00+dTime*(i+1),100,20,20)
				
